# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by ..\lib\unicore\mktables from the Unicode
# database, Version 9.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V38
2385
2387
2404
2406
3200
3204
3205
3213
3214
3217
3218
3241
3242
3252
3253
3258
3260
3269
3270
3273
3274
3278
3285
3287
3294
3295
3296
3300
3302
3312
3313
3315
7386
7387
7413
7414
43056
43062
END
