{
  am_pm_abbreviated => [
    "go.\N{U+00f6}\N{U+0148}",
    "go.so\N{U+0148}",
  ],
  available_formats => {
    Bh => "h B",
    Bhm => "h:mm B",
    Bhms => "h:mm:ss B",
    E => "ccc",
    EBhm => "E h:mm B",
    EBhms => "E h:mm:ss B",
    EHm => "E HH:mm",
    EHms => "E HH:mm:ss",
    Ed => "d E",
    Ehm => "E h:mm a",
    Ehms => "E h:mm:ss a",
    Gy => "G y",
    GyMMM => "G y MMM",
    GyMMMEd => "G y MMM d, E",
    GyMMMd => "G y MMM d",
    H => "HH",
    Hm => "HH:mm",
    Hms => "HH:mm:ss",
    Hmsv => "HH:mm:ss v",
    Hmv => "HH:mm v",
    M => "L",
    MEd => "dd.MM E",
    MMM => "LLL",
    MMMEd => "d MMM E",
    MMMMEd => "d MMMM E",
    "MMMMW-count-one" => "'hepde' W, MMMM",
    "MMMMW-count-other" => "'hepde' W, MMMM",
    MMMMd => "d MMMM",
    MMMd => "d MMM",
    Md => "dd.MM",
    d => "d",
    h => "h a",
    hm => "h:mm a",
    hms => "h:mm:ss a",
    hmsv => "h:mm:ss a v",
    hmv => "h:mm a v",
    mmss => "mm:ss",
    ms => "mm:ss",
    y => "y",
    yM => "MM.y",
    yMEd => "dd.MM.y E",
    yMMM => "MMM y",
    yMMMEd => "d MMM y E",
    yMMMM => "MMMM y",
    yMMMd => "d MMM y",
    yMd => "dd.MM.y",
    yQQQ => "y QQQ",
    yQQQQ => "y QQQQ",
    "yw-count-one" => "'hepde' w, Y",
    "yw-count-other" => "'hepde' w, Y",
  },
  code => "tk-TM",
  date_format_full => "d MMMM y EEEE",
  date_format_long => "d MMMM y",
  date_format_medium => "d MMM y",
  date_format_short => "dd.MM.y",
  datetime_format_full => "{1} {0}",
  datetime_format_long => "{1} {0}",
  datetime_format_medium => "{1} {0}",
  datetime_format_short => "{1} {0}",
  day_format_abbreviated => [
    "du\N{U+015f}",
    "si\N{U+015f}",
    "\N{U+00e7}ar",
    "pen",
    "ann",
    "\N{U+015f}en",
    "\N{U+00fd}ek",
  ],
  day_format_narrow => [
    "D",
    "S",
    "\N{U+00c7}",
    "P",
    "A",
    "\N{U+015e}",
    "\N{U+00dd}",
  ],
  day_format_wide => [
    "du\N{U+015f}enbe",
    "si\N{U+015f}enbe",
    "\N{U+00e7}ar\N{U+015f}enbe",
    "pen\N{U+015f}enbe",
    "anna",
    "\N{U+015f}enbe",
    "\N{U+00fd}ek\N{U+015f}enbe",
  ],
  day_stand_alone_abbreviated => [
    "Du\N{U+015f}",
    "Si\N{U+015f}",
    "\N{U+00c7}ar",
    "Pen",
    "Ann",
    "\N{U+015e}en",
    "\N{U+00dd}ek",
  ],
  day_stand_alone_narrow => [
    "D",
    "S",
    "\N{U+00c7}",
    "P",
    "A",
    "\N{U+015e}",
    "\N{U+00dd}",
  ],
  day_stand_alone_wide => [
    "Du\N{U+015f}enbe",
    "Si\N{U+015f}enbe",
    "\N{U+00c7}ar\N{U+015f}enbe",
    "Pen\N{U+015f}enbe",
    "Anna",
    "\N{U+015e}enbe",
    "\N{U+00dd}ek\N{U+015f}enbe",
  ],
  era_abbreviated => [
    "B.e.\N{U+00f6}\N{U+0148}",
    "B.e.",
  ],
  era_narrow => [
    "B.e.\N{U+00f6}\N{U+0148}",
    "B.e.",
  ],
  era_wide => [
    "Isadan \N{U+00f6}\N{U+0148}",
    "Isadan so\N{U+0148}",
  ],
  first_day_of_week => 1,
  glibc_date_1_format => "%a %b %e %H:%M:%S %Z %Y",
  glibc_date_format => "%d.%m.%Y",
  glibc_datetime_format => "%d.%m.%Y %T",
  glibc_time_12_format => "%I:%M:%S %p",
  glibc_time_format => "%T",
  language => "Turkmen",
  month_format_abbreviated => [
    "\N{U+00fd}an",
    "few",
    "mart",
    "apr",
    "ma\N{U+00fd}",
    "i\N{U+00fd}un",
    "i\N{U+00fd}ul",
    "awg",
    "sen",
    "okt",
    "no\N{U+00fd}",
    "dek",
  ],
  month_format_narrow => [
    "\N{U+00dd}",
    "F",
    "M",
    "A",
    "M",
    "I",
    "I",
    "A",
    "S",
    "O",
    "N",
    "D",
  ],
  month_format_wide => [
    "\N{U+00fd}anwar",
    "fewral",
    "mart",
    "aprel",
    "ma\N{U+00fd}",
    "i\N{U+00fd}un",
    "i\N{U+00fd}ul",
    "awgust",
    "sent\N{U+00fd}abr",
    "okt\N{U+00fd}abr",
    "no\N{U+00fd}abr",
    "dekabr",
  ],
  month_stand_alone_abbreviated => [
    "\N{U+00dd}an",
    "Few",
    "Mart",
    "Apr",
    "Ma\N{U+00fd}",
    "I\N{U+00fd}un",
    "I\N{U+00fd}ul",
    "Awg",
    "Sen",
    "Okt",
    "No\N{U+00fd}",
    "Dek",
  ],
  month_stand_alone_narrow => [
    "\N{U+00dd}",
    "F",
    "M",
    "A",
    "M",
    "I",
    "I",
    "A",
    "S",
    "O",
    "N",
    "D",
  ],
  month_stand_alone_wide => [
    "\N{U+00dd}anwar",
    "Fewral",
    "Mart",
    "Aprel",
    "Ma\N{U+00fd}",
    "I\N{U+00fd}un",
    "I\N{U+00fd}ul",
    "Awgust",
    "Sent\N{U+00fd}abr",
    "Okt\N{U+00fd}abr",
    "No\N{U+00fd}abr",
    "Dekabr",
  ],
  name => "Turkmen Turkmenistan",
  native_language => "t\N{U+00fc}rkmen dili",
  native_name => "t\N{U+00fc}rkmen dili T\N{U+00fc}rkmenistan",
  native_script => undef,
  native_territory => "T\N{U+00fc}rkmenistan",
  native_variant => undef,
  quarter_format_abbreviated => [
    "1\N{U+00c7}",
    "2\N{U+00c7}",
    "3\N{U+00c7}",
    "4\N{U+00c7}",
  ],
  quarter_format_narrow => [
    1,
    2,
    3,
    4,
  ],
  quarter_format_wide => [
    "1-nji \N{U+00e7}\N{U+00e4}r\N{U+00fd}ek",
    "2-nji \N{U+00e7}\N{U+00e4}r\N{U+00fd}ek",
    "3-nji \N{U+00e7}\N{U+00e4}r\N{U+00fd}ek",
    "4-nji \N{U+00e7}\N{U+00e4}r\N{U+00fd}ek",
  ],
  quarter_stand_alone_abbreviated => [
    "1\N{U+00c7}",
    "2\N{U+00c7}",
    "3\N{U+00c7}",
    "Q4",
  ],
  quarter_stand_alone_narrow => [
    1,
    2,
    3,
    4,
  ],
  quarter_stand_alone_wide => [
    "1-nji \N{U+00e7}\N{U+00e4}r\N{U+00fd}ek",
    "2-nji \N{U+00e7}\N{U+00e4}r\N{U+00fd}ek",
    "3-nji \N{U+00e7}\N{U+00e4}r\N{U+00fd}ek",
    "4-nji \N{U+00e7}\N{U+00e4}r\N{U+00fd}ek",
  ],
  script => undef,
  territory => "Turkmenistan",
  time_format_full => "HH:mm:ss zzzz",
  time_format_long => "HH:mm:ss z",
  time_format_medium => "HH:mm:ss",
  time_format_short => "HH:mm",
  variant => undef,
  version => 33,
}
