{
  am_pm_abbreviated => [
    "ib",
    "eb",
  ],
  available_formats => {
    Bh => "h B",
    Bhm => "h:mm B",
    Bhms => "h:mm:ss B",
    E => "ccc",
    EBhm => "E h:mm B",
    EBhms => "E h:mm:ss B",
    EHm => "E HH:mm",
    EHms => "E HH:mm:ss",
    Ed => "d E",
    Ehm => "E h:mm a",
    Ehms => "E h:mm:ss a",
    Gy => "y G",
    GyMMM => "MMM y G",
    GyMMMEd => "E d MMM y G",
    GyMMMd => "d MMM y G",
    H => "HH",
    Hm => "HH:mm",
    Hms => "HH:mm:ss",
    Hmsv => "HH:mm:ss v",
    Hmv => "HH:mm v",
    M => "L",
    MEd => "E d.M",
    MMM => "LLL",
    MMMEd => "E d MMM",
    "MMMMW-count-one" => "MMMM:a 'vahkku' W",
    "MMMMW-count-other" => "MMM:a 'vahkku' W",
    "MMMMW-count-two" => "MMMM:a 'vahkku' W",
    MMMMd => "d MMMM",
    MMMd => "d MMM",
    Md => "d/M",
    d => "d",
    h => "h a",
    hm => "h:mm a",
    hms => "h:mm:ss a",
    hmsv => "h:mm:ss a v",
    hmv => "h:mm a v",
    ms => "mm:ss",
    y => "y",
    yM => "MM.y",
    yMEd => "E dd.MM.y",
    yMMM => "MMM y",
    yMMMEd => "E d MMM y",
    yMMMM => "MMMM y",
    yMMMd => "d MMM y",
    yMd => "dd.MM.y",
    yQQQ => "QQQ y",
    yQQQQ => "QQQQ y",
    "yw-count-one" => "'week' w 'of' Y",
    "yw-count-other" => "'week' w 'of' Y",
    "yw-count-two" => "'week' w 'of' Y",
  },
  code => "se-FI",
  date_format_full => "EEEE d MMMM y",
  date_format_long => "d MMMM y",
  date_format_medium => "d MMM y",
  date_format_short => "dd.MM.y",
  datetime_format_full => "{1} {0}",
  datetime_format_long => "{1} {0}",
  datetime_format_medium => "{1} {0}",
  datetime_format_short => "{1} {0}",
  day_format_abbreviated => [
    "m\N{U+00e1}",
    "di",
    "ga",
    "du",
    "be",
    "l\N{U+00e1}",
    "so",
  ],
  day_format_narrow => [
    "M",
    "D",
    "G",
    "D",
    "B",
    "L",
    "S",
  ],
  day_format_wide => [
    "m\N{U+00e1}nnodat",
    "disdat",
    "gaskavahkku",
    "duorastat",
    "bearjadat",
    "l\N{U+00e1}vvordat",
    "sotnabeaivi",
  ],
  day_stand_alone_abbreviated => [
    "m\N{U+00e1}",
    "di",
    "ga",
    "du",
    "be",
    "l\N{U+00e1}",
    "so",
  ],
  day_stand_alone_narrow => [
    "M",
    "D",
    "G",
    "D",
    "B",
    "L",
    "S",
  ],
  day_stand_alone_wide => [
    "m\N{U+00e1}nnodat",
    "disdat",
    "gaskavahkku",
    "duorastat",
    "bearjadat",
    "l\N{U+00e1}vvordat",
    "sotnabeaivi",
  ],
  era_abbreviated => [
    "oKr.",
    "mKr.",
  ],
  era_narrow => [
    "oKr.",
    "mKr.",
  ],
  era_wide => [
    "ovdal Kristusa",
    "ma\N{U+014b}\N{U+014b}el Kristusa",
  ],
  first_day_of_week => 1,
  glibc_date_1_format => "%a %b %e %H:%M:%S %Z %Y",
  glibc_date_format => "%m/%d/%y",
  glibc_datetime_format => "%a %b %e %H:%M:%S %Y",
  glibc_time_12_format => "%I:%M:%S %p",
  glibc_time_format => "%H:%M:%S",
  language => "Northern Sami",
  month_format_abbreviated => [
    "o\N{U+0111}\N{U+0111}j",
    "guov",
    "njuk",
    "cuo\N{U+014b}",
    "mies",
    "geas",
    "suoi",
    "borg",
    "\N{U+010d}ak\N{U+010d}",
    "golg",
    "sk\N{U+00e1}b",
    "juov",
  ],
  month_format_narrow => [
    "O",
    "G",
    "N",
    "C",
    "M",
    "G",
    "S",
    "B",
    "\N{U+010c}",
    "G",
    "S",
    "J",
  ],
  month_format_wide => [
    "o\N{U+0111}\N{U+0111}ajagem\N{U+00e1}nnu",
    "guovvam\N{U+00e1}nnu",
    "njuk\N{U+010d}am\N{U+00e1}nnu",
    "cuo\N{U+014b}om\N{U+00e1}nnu",
    "miessem\N{U+00e1}nnu",
    "geassem\N{U+00e1}nnu",
    "suoidnem\N{U+00e1}nnu",
    "borgem\N{U+00e1}nnu",
    "\N{U+010d}ak\N{U+010d}am\N{U+00e1}nnu",
    "golggotm\N{U+00e1}nnu",
    "sk\N{U+00e1}bmam\N{U+00e1}nnu",
    "juovlam\N{U+00e1}nnu",
  ],
  month_stand_alone_abbreviated => [
    "o\N{U+0111}\N{U+0111}j",
    "guov",
    "njuk",
    "cuo\N{U+014b}",
    "mies",
    "geas",
    "suoi",
    "borg",
    "\N{U+010d}ak\N{U+010d}",
    "golg",
    "sk\N{U+00e1}b",
    "juov",
  ],
  month_stand_alone_narrow => [
    "O",
    "G",
    "N",
    "C",
    "M",
    "G",
    "S",
    "B",
    "\N{U+010c}",
    "G",
    "S",
    "J",
  ],
  month_stand_alone_wide => [
    "o\N{U+0111}\N{U+0111}ajagem\N{U+00e1}nnu",
    "guovvam\N{U+00e1}nnu",
    "njuk\N{U+010d}am\N{U+00e1}nnu",
    "cuo\N{U+014b}om\N{U+00e1}nnu",
    "miessem\N{U+00e1}nnu",
    "geassem\N{U+00e1}nnu",
    "suoidnem\N{U+00e1}nnu",
    "borgem\N{U+00e1}nnu",
    "\N{U+010d}ak\N{U+010d}am\N{U+00e1}nnu",
    "golggotm\N{U+00e1}nnu",
    "sk\N{U+00e1}bmam\N{U+00e1}nnu",
    "juovlam\N{U+00e1}nnu",
  ],
  name => "Northern Sami Finland",
  native_language => "davvis\N{U+00e1}megiella",
  native_name => "davvis\N{U+00e1}megiella Suopma",
  native_script => undef,
  native_territory => "Suopma",
  native_variant => undef,
  quarter_format_abbreviated => [
    "1Q",
    "2Q",
    "3Q",
    "4Q",
  ],
  quarter_format_narrow => [
    1,
    2,
    3,
    4,
  ],
  quarter_format_wide => [
    "1. njealj\N{U+00e1}das",
    "2. njealj\N{U+00e1}das",
    "3. njealj\N{U+00e1}das",
    "4. njealj\N{U+00e1}das",
  ],
  quarter_stand_alone_abbreviated => [
    "Q1",
    "2Q",
    "3Q",
    "4Q",
  ],
  quarter_stand_alone_narrow => [
    1,
    2,
    3,
    4,
  ],
  quarter_stand_alone_wide => [
    "1. njealj\N{U+00e1}das",
    "2. njealj\N{U+00e1}das",
    "3. njealj\N{U+00e1}das",
    "4. njealj\N{U+00e1}das",
  ],
  script => undef,
  territory => "Finland",
  time_format_full => "HH:mm:ss zzzz",
  time_format_long => "HH:mm:ss z",
  time_format_medium => "HH:mm:ss",
  time_format_short => "HH:mm",
  variant => undef,
  version => 33,
}
