# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by ..\lib\unicore\mktables from the Unicode
# database, Version 9.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V28
2432
2436
2437
2445
2447
2449
2451
2473
2474
2481
2482
2483
2486
2490
2492
2501
2503
2505
2507
2511
2519
2520
2524
2526
2527
2532
2534
2556
END
