# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by ..\lib\unicore\mktables from the Unicode
# database, Version 9.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V48
170
171
178
180
185
187
688
697
736
741
4348
4349
7468
7471
7472
7483
7484
7502
7503
7522
7544
7545
7579
7616
8304
8306
8308
8320
8480
8481
8482
8483
11389
11390
11631
11632
12690
12704
42652
42654
42864
42865
43000
43002
43868
43872
127338
127340
END
