# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by ..\lib\unicore\mktables from the Unicode
# database, Version 9.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V32
2385
2387
2404
2406
3072
3076
3077
3085
3086
3089
3090
3113
3114
3130
3133
3141
3142
3145
3146
3150
3157
3159
3160
3163
3168
3172
3174
3184
3192
3200
7386
7387
END
