# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by ..\lib\unicore\mktables from the Unicode
# database, Version 9.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V30
3642
3643
3662
3663
3972
3973
4154
4155
5908
5909
5940
5941
6097
6098
7082
7083
7154
7156
43014
43015
43347
43348
44013
44014
69940
69941
70378
70379
71467
71468
END
