# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by ..\lib\unicore\mktables from the Unicode
# database, Version 9.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V34
45
46
1418
1419
1470
1471
5120
5121
6150
6151
8208
8214
11799
11800
11802
11803
11834
11836
11840
11841
12316
12317
12336
12337
12448
12449
65073
65075
65112
65113
65123
65124
65293
65294
END
