# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by ..\lib\unicore\mktables from the Unicode
# database, Version 9.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V32
11904
11930
11931
12020
12032
12246
12293
12294
12295
12296
12321
12330
12344
12348
13312
19894
19968
40918
63744
64110
64112
64218
131072
173783
173824
177973
177984
178206
178208
183970
194560
195102
END
