{
  am_pm_abbreviated => [
    "a. m.",
    "p. m.",
  ],
  available_formats => {
    Bh => "h B",
    Bhm => "h:mm B",
    Bhms => "h:mm:ss B",
    E => "ccc",
    EBhm => "E h:mm B",
    EBhms => "E h:mm:ss B",
    EHm => "E H:mm",
    EHms => "E H:mm:ss",
    Ed => "E d",
    Ehm => "E h:mm a",
    Ehms => "E h:mm:ss a",
    Gy => "y G",
    GyMMM => "LLL y G",
    GyMMMEd => "E, d MMM 'de' y G",
    GyMMMM => "LLLL 'de' y G",
    GyMMMMEd => "E, d MMMM 'de' y G",
    GyMMMMd => "d MMMM 'de' y G",
    GyMMMd => "d MMM 'de' y G",
    H => "H",
    Hm => "H:mm",
    Hms => "H:mm:ss",
    Hmsv => "H:mm:ss v",
    Hmv => "H:mm v",
    M => "L",
    MEd => "E d/M",
    MMM => "LLL",
    MMMEd => "E, d MMM",
    MMMMEd => "E, d MMMM",
    "MMMMW-count-one" => "'setmana' W MMMM",
    "MMMMW-count-other" => "'setmana' W MMMM",
    MMMMd => "d MMMM",
    MMMd => "d MMM",
    Md => "d/M",
    d => "d",
    h => "h a",
    hm => "h:mm a",
    hms => "h:mm:ss a",
    hmsv => "h:mm:ss a v",
    hmv => "h:mm a v",
    ms => "mm:ss",
    y => "y",
    yM => "M/y",
    yMEd => "E, d/M/y",
    yMMM => "LLL 'de' y",
    yMMMEd => "E, d MMM y",
    yMMMM => "LLLL 'de' y",
    yMMMMEd => "E, d MMMM 'de' y",
    yMMMMd => "d MMMM 'de' y",
    yMMMd => "d MMM 'de' y",
    yMd => "d/M/y",
    yQQQ => "QQQ y",
    yQQQQ => "QQQQ y",
    "yw-count-one" => "'setmana' w 'de' Y",
    "yw-count-other" => "'setmana' w 'de' Y",
  },
  code => "ca-ES-VALENCIA",
  date_format_full => "EEEE, d MMMM 'de' y",
  date_format_long => "d MMMM 'de' y",
  date_format_medium => "d MMM y",
  date_format_short => "d/M/yy",
  datetime_format_full => "{1} 'a' 'les' {0}",
  datetime_format_long => "{1} 'a' 'les' {0}",
  datetime_format_medium => "{1}, {0}",
  datetime_format_short => "{1} {0}",
  day_format_abbreviated => [
    "dl.",
    "dt.",
    "dc.",
    "dj.",
    "dv.",
    "ds.",
    "dg.",
  ],
  day_format_narrow => [
    "dl",
    "dt",
    "dc",
    "dj",
    "dv",
    "ds",
    "dg",
  ],
  day_format_wide => [
    "dilluns",
    "dimarts",
    "dimecres",
    "dijous",
    "divendres",
    "dissabte",
    "diumenge",
  ],
  day_stand_alone_abbreviated => [
    "dl.",
    "dt.",
    "dc.",
    "dj.",
    "dv.",
    "ds.",
    "dg.",
  ],
  day_stand_alone_narrow => [
    "dl",
    "dt",
    "dc",
    "dj",
    "dv",
    "ds",
    "dg",
  ],
  day_stand_alone_wide => [
    "dilluns",
    "dimarts",
    "dimecres",
    "dijous",
    "divendres",
    "dissabte",
    "diumenge",
  ],
  era_abbreviated => [
    "aC",
    "dC",
  ],
  era_narrow => [
    "aC",
    "dC",
  ],
  era_wide => [
    "abans de Crist",
    "despr\N{U+00e9}s de Crist",
  ],
  first_day_of_week => 1,
  glibc_date_1_format => "%a %b %e %H:%M:%S %Z %Y",
  glibc_date_format => "%d/%m/%y",
  glibc_datetime_format => "%a %d %b %Y %T %Z",
  glibc_time_12_format => "%I:%M:%S %p",
  glibc_time_format => "%T",
  language => "Catalan",
  month_format_abbreviated => [
    "de gen.",
    "de febr.",
    "de mar\N{U+00e7}",
    "d\N{U+2019}abr.",
    "de maig",
    "de juny",
    "de jul.",
    "d\N{U+2019}ag.",
    "de set.",
    "d\N{U+2019}oct.",
    "de nov.",
    "de des.",
  ],
  month_format_narrow => [
    "GN",
    "FB",
    "M\N{U+00c7}",
    "AB",
    "MG",
    "JN",
    "JL",
    "AG",
    "ST",
    "OC",
    "NV",
    "DS",
  ],
  month_format_wide => [
    "de gener",
    "de febrer",
    "de mar\N{U+00e7}",
    "d\N{U+2019}abril",
    "de maig",
    "de juny",
    "de juliol",
    "d\N{U+2019}agost",
    "de setembre",
    "d\N{U+2019}octubre",
    "de novembre",
    "de desembre",
  ],
  month_stand_alone_abbreviated => [
    "gen.",
    "febr.",
    "mar\N{U+00e7}",
    "abr.",
    "maig",
    "juny",
    "jul.",
    "ag.",
    "set.",
    "oct.",
    "nov.",
    "des.",
  ],
  month_stand_alone_narrow => [
    "GN",
    "FB",
    "M\N{U+00c7}",
    "AB",
    "MG",
    "JN",
    "JL",
    "AG",
    "ST",
    "OC",
    "NV",
    "DS",
  ],
  month_stand_alone_wide => [
    "gener",
    "febrer",
    "mar\N{U+00e7}",
    "abril",
    "maig",
    "juny",
    "juliol",
    "agost",
    "setembre",
    "octubre",
    "novembre",
    "desembre",
  ],
  name => "Catalan Spain Valencian",
  native_language => "catal\N{U+00e0}",
  native_name => "catal\N{U+00e0} Espanya valenci\N{U+00e0}",
  native_script => undef,
  native_territory => "Espanya",
  native_variant => "valenci\N{U+00e0}",
  quarter_format_abbreviated => [
    "1T",
    "2T",
    "3T",
    "4T",
  ],
  quarter_format_narrow => [
    1,
    2,
    3,
    4,
  ],
  quarter_format_wide => [
    "1r trimestre",
    "2n trimestre",
    "3r trimestre",
    "4t trimestre",
  ],
  quarter_stand_alone_abbreviated => [
    "1T",
    "2T",
    "3T",
    "4T",
  ],
  quarter_stand_alone_narrow => [
    1,
    2,
    3,
    4,
  ],
  quarter_stand_alone_wide => [
    "1r trimestre",
    "2n trimestre",
    "3r trimestre",
    "4t trimestre",
  ],
  script => undef,
  territory => "Spain",
  time_format_full => "H:mm:ss zzzz",
  time_format_long => "H:mm:ss z",
  time_format_medium => "H:mm:ss",
  time_format_short => "H:mm",
  variant => "Valencian",
  version => 33,
}
