{
  am_pm_abbreviated => [
    "man\N{U+00e1}",
    "kug\N{U+00fa}",
  ],
  available_formats => {
    Bh => "h B",
    Bhm => "h:mm B",
    Bhms => "h:mm:ss B",
    E => "ccc",
    EBhm => "E h:mm B",
    EBhms => "E h:mm:ss B",
    EHm => "E HH:mm",
    EHms => "E HH:mm:ss",
    Ed => "E d",
    Ehm => "E h:mm a",
    Ehms => "E h:mm:ss a",
    Gy => "G y",
    GyMMM => "G y MMM",
    GyMMMEd => "G y MMM d, E",
    GyMMMd => "G y MMM d",
    H => "HH",
    Hm => "HH:mm",
    Hms => "HH:mm:ss",
    Hmsv => "HH:mm:ss v",
    Hmv => "HH:mm v",
    M => "L",
    MEd => "E d/M",
    MMM => "LLL",
    MMMEd => "E d MMM",
    "MMMMW-count-other" => "'week' W 'of' MMMM",
    MMMMd => "MMMM d",
    MMMd => "d MMM",
    Md => "d/M",
    d => "d",
    h => "h a",
    hm => "h:mm a",
    hms => "h:mm:ss a",
    hmsv => "h:mm:ss a v",
    hmv => "h:mm a v",
    ms => "m:ss",
    y => "y",
    yM => "M/y",
    yMEd => "E d/M/y",
    yMMM => "MMM y",
    yMMMEd => "E d MMM y",
    yMMMM => "y MMMM",
    yMMMd => "d MMM y",
    yMd => "d/M/y",
    yQQQ => "QQQ y",
    yQQQQ => "QQQQ y",
    "yw-count-other" => "'week' w 'of' Y",
  },
  code => "nmg",
  date_format_full => "EEEE d MMMM y",
  date_format_long => "d MMMM y",
  date_format_medium => "d MMM y",
  date_format_short => "d/M/y",
  datetime_format_full => "{1} {0}",
  datetime_format_long => "{1} {0}",
  datetime_format_medium => "{1} {0}",
  datetime_format_short => "{1} {0}",
  day_format_abbreviated => [
    "m\N{U+0254}\N{U+0301}n",
    "smb",
    "sml",
    "smn",
    "mbs",
    "sas",
    "s\N{U+0254}\N{U+0301}n",
  ],
  day_format_narrow => [
    "m",
    "s",
    "s",
    "s",
    "m",
    "s",
    "s",
  ],
  day_format_wide => [
    "m\N{U+0254}\N{U+0301}nd\N{U+0254}",
    "s\N{U+0254}\N{U+0301}nd\N{U+0254} maf\N{U+00fa} m\N{U+00e1}ba",
    "s\N{U+0254}\N{U+0301}nd\N{U+0254} maf\N{U+00fa} m\N{U+00e1}lal",
    "s\N{U+0254}\N{U+0301}nd\N{U+0254} maf\N{U+00fa} m\N{U+00e1}na",
    "mab\N{U+00e1}g\N{U+00e1} m\N{U+00e1} sukul",
    "s\N{U+00e1}sadi",
    "s\N{U+0254}\N{U+0301}nd\N{U+0254}",
  ],
  day_stand_alone_abbreviated => [
    "m\N{U+0254}\N{U+0301}n",
    "smb",
    "sml",
    "smn",
    "mbs",
    "sas",
    "s\N{U+0254}\N{U+0301}n",
  ],
  day_stand_alone_narrow => [
    "m",
    "s",
    "s",
    "s",
    "m",
    "s",
    "s",
  ],
  day_stand_alone_wide => [
    "m\N{U+0254}\N{U+0301}nd\N{U+0254}",
    "s\N{U+0254}\N{U+0301}nd\N{U+0254} maf\N{U+00fa} m\N{U+00e1}ba",
    "s\N{U+0254}\N{U+0301}nd\N{U+0254} maf\N{U+00fa} m\N{U+00e1}lal",
    "s\N{U+0254}\N{U+0301}nd\N{U+0254} maf\N{U+00fa} m\N{U+00e1}na",
    "mab\N{U+00e1}g\N{U+00e1} m\N{U+00e1} sukul",
    "s\N{U+00e1}sadi",
    "s\N{U+0254}\N{U+0301}nd\N{U+0254}",
  ],
  era_abbreviated => [
    "BL",
    "PB",
  ],
  era_narrow => [
    "BL",
    "PB",
  ],
  era_wide => [
    "B\N{U+00f3} Lahl\N{U+025b}\N{U+0304}",
    "Pfi\N{U+025b} Bur\N{U+012b}",
  ],
  first_day_of_week => 1,
  glibc_date_1_format => "%a %b %e %H:%M:%S %Z %Y",
  glibc_date_format => "%m/%d/%y",
  glibc_datetime_format => "%a %b %e %H:%M:%S %Y",
  glibc_time_12_format => "%I:%M:%S %p",
  glibc_time_format => "%H:%M:%S",
  language => "Kwasio",
  month_format_abbreviated => [
    "ng1",
    "ng2",
    "ng3",
    "ng4",
    "ng5",
    "ng6",
    "ng7",
    "ng8",
    "ng9",
    "ng10",
    "ng11",
    "kris",
  ],
  month_format_narrow => [
    1,
    2,
    3,
    4,
    5,
    6,
    7,
    8,
    9,
    10,
    11,
    12,
  ],
  month_format_wide => [
    "ngw\N{U+025b}n mat\N{U+00e1}hra",
    "ngw\N{U+025b}n \N{U+0144}mba",
    "ngw\N{U+025b}n \N{U+0144}lal",
    "ngw\N{U+025b}n \N{U+0144}na",
    "ngw\N{U+025b}n \N{U+0144}tan",
    "ngw\N{U+025b}n \N{U+0144}tu\N{U+00f3}",
    "ngw\N{U+025b}n h\N{U+025b}mbu\N{U+025b}r\N{U+00ed}",
    "ngw\N{U+025b}n l\N{U+0254}mbi",
    "ngw\N{U+025b}n r\N{U+025b}bvu\N{U+00e2}",
    "ngw\N{U+025b}n wum",
    "ngw\N{U+025b}n wum nav\N{U+01d4}r",
    "kr\N{U+00ed}simin",
  ],
  month_stand_alone_abbreviated => [
    "ng1",
    "ng2",
    "ng3",
    "ng4",
    "ng5",
    "ng6",
    "ng7",
    "ng8",
    "ng9",
    "ng10",
    "ng11",
    "kris",
  ],
  month_stand_alone_narrow => [
    1,
    2,
    3,
    4,
    5,
    6,
    7,
    8,
    9,
    10,
    11,
    12,
  ],
  month_stand_alone_wide => [
    "ngw\N{U+025b}n mat\N{U+00e1}hra",
    "ngw\N{U+025b}n \N{U+0144}mba",
    "ngw\N{U+025b}n \N{U+0144}lal",
    "ngw\N{U+025b}n \N{U+0144}na",
    "ngw\N{U+025b}n \N{U+0144}tan",
    "ngw\N{U+025b}n \N{U+0144}tu\N{U+00f3}",
    "ngw\N{U+025b}n h\N{U+025b}mbu\N{U+025b}r\N{U+00ed}",
    "ngw\N{U+025b}n l\N{U+0254}mbi",
    "ngw\N{U+025b}n r\N{U+025b}bvu\N{U+00e2}",
    "ngw\N{U+025b}n wum",
    "ngw\N{U+025b}n wum nav\N{U+01d4}r",
    "kr\N{U+00ed}simin",
  ],
  name => "Kwasio",
  native_language => "nmg",
  native_name => "nmg",
  native_script => undef,
  native_territory => undef,
  native_variant => undef,
  quarter_format_abbreviated => [
    "T1",
    "T2",
    "T3",
    "T4",
  ],
  quarter_format_narrow => [
    1,
    2,
    3,
    4,
  ],
  quarter_format_wide => [
    "Tind\N{U+025b} nv\N{U+00fa}r",
    "Tind\N{U+025b} \N{U+0144}mba",
    "Tind\N{U+025b} \N{U+0144}lal",
    "Tind\N{U+025b} \N{U+0144}na",
  ],
  quarter_stand_alone_abbreviated => [
    "T1",
    "T2",
    "T3",
    "T4",
  ],
  quarter_stand_alone_narrow => [
    1,
    2,
    3,
    4,
  ],
  quarter_stand_alone_wide => [
    "Tind\N{U+025b} nv\N{U+00fa}r",
    "Tind\N{U+025b} \N{U+0144}mba",
    "Tind\N{U+025b} \N{U+0144}lal",
    "Tind\N{U+025b} \N{U+0144}na",
  ],
  script => undef,
  territory => undef,
  time_format_full => "HH:mm:ss zzzz",
  time_format_long => "HH:mm:ss z",
  time_format_medium => "HH:mm:ss",
  time_format_short => "HH:mm",
  variant => undef,
  version => 33,
}
