# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by ..\lib\unicore\mktables from the Unicode
# database, Version 9.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V54
768
880
1157
1159
1611
1622
1648
1649
2385
2387
6832
6847
7376
7379
7380
7393
7394
7401
7405
7406
7412
7413
7416
7418
7616
7670
7675
7680
8204
8206
8400
8433
12330
12334
12441
12443
65024
65040
65056
65070
66045
66046
66272
66273
119143
119146
119163
119171
119173
119180
119210
119214
917760
918000
END
