# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by ..\lib\unicore\mktables from the Unicode
# database, Version 9.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V50
12353
12354
12355
12356
12357
12358
12359
12360
12361
12362
12387
12388
12419
12420
12421
12422
12423
12424
12430
12431
12437
12439
12449
12450
12451
12452
12453
12454
12455
12456
12457
12458
12483
12484
12515
12516
12517
12518
12519
12520
12526
12527
12533
12535
12540
12541
12784
12800
65383
65393
END
