.str{color:#50a14f}.kwd{color:#a626a4}.com{color:#a0a1a7}.typ,.dec{color:#c18401} .lit,.atn{color:#0184bc} .pun,.opn,.clo,.pln{color:#383a42} .tag,.var,.fun{color:#e45649} .atv{color:#4078f2} pre.prettyprint{border:1px solid #c1c1c1;padding:1em}
