{
  am_pm_abbreviated => [
    "prije podne",
    "po podne",
  ],
  available_formats => {
    Bh => "h B",
    Bhm => "hh:mm B",
    Bhms => "hh:mm:ss B",
    E => "E",
    EBhm => "E hh:mm B",
    EBhms => "E hh:mm:ss B",
    EHm => "E HH:mm",
    EHms => "E HH:mm:ss",
    Ed => "E d.",
    Ehm => "E h:mm a",
    Ehms => "E h:mm:ss a",
    Gy => "y. G",
    GyMMM => "MMM y. G",
    GyMMMEd => "E, d. MMM y. G",
    GyMMMd => "d. MMM y. G",
    H => "HH",
    Hm => "HH:mm",
    Hms => "HH:mm:ss",
    Hmsv => "HH:mm:ss v",
    Hmv => "HH:mm v",
    M => "L",
    MEd => "E, d.M.",
    MMM => "LLL",
    MMMEd => "E d. MMM",
    MMMMEd => "E, d. MMMM",
    "MMMMW-count-few" => "W. 'sjedmica' 'u' MMM",
    "MMMMW-count-one" => "W. 'sjedmica' 'u' MMM",
    "MMMMW-count-other" => "W. 'sjedmica' 'u' MMM",
    MMMMd => "d. MMMM",
    MMMd => "d. MMM",
    MMMdd => "dd.MMM",
    MMdd => "dd.MM.",
    Md => "d.M.",
    d => "d",
    h => "h a",
    hm => "h:mm a",
    hms => "h:mm:ss a",
    hmsv => "h:mm:ss a v",
    hmv => "h:mm a v",
    ms => "mm:ss",
    y => "y.",
    yM => "M.y.",
    yMEd => "E, d.M.y.",
    yMM => "MM.y.",
    yMMM => "MMM y.",
    yMMMEd => "E, d. MMM y.",
    yMMMM => "MMMM y.",
    yMMMd => "d. MMM y.",
    yMMdd => "dd.MM.y.",
    yMd => "d.M.y.",
    yQQQ => "QQQ y.",
    yQQQQ => "QQQQ y.",
    "yw-count-few" => "w. 'sjedmica' 'u' y.",
    "yw-count-one" => "w. 'sjedmica' 'u' y.",
    "yw-count-other" => "w. 'sjedmica' 'u' y.",
  },
  code => "sr-Latn-ME",
  date_format_full => "EEEE, dd. MMMM y.",
  date_format_long => "dd. MMMM y.",
  date_format_medium => "dd.MM.y.",
  date_format_short => "d.M.yy.",
  datetime_format_full => "{1} {0}",
  datetime_format_long => "{1} {0}",
  datetime_format_medium => "{1} {0}",
  datetime_format_short => "{1} {0}",
  day_format_abbreviated => [
    "pon.",
    "ut.",
    "sr.",
    "\N{U+010d}et.",
    "pet.",
    "sub.",
    "ned.",
  ],
  day_format_narrow => [
    "p",
    "u",
    "s",
    "\N{U+010d}",
    "p",
    "s",
    "n",
  ],
  day_format_wide => [
    "ponedeljak",
    "utorak",
    "srijeda",
    "\N{U+010d}etvrtak",
    "petak",
    "subota",
    "nedjelja",
  ],
  day_stand_alone_abbreviated => [
    "pon.",
    "ut.",
    "sr.",
    "\N{U+010d}et.",
    "pet.",
    "sub.",
    "ned.",
  ],
  day_stand_alone_narrow => [
    "p",
    "u",
    "s",
    "\N{U+010d}",
    "p",
    "s",
    "n",
  ],
  day_stand_alone_wide => [
    "ponedeljak",
    "utorak",
    "srijeda",
    "\N{U+010d}etvrtak",
    "petak",
    "subota",
    "nedjelja",
  ],
  era_abbreviated => [
    "p. n. e.",
    "n. e.",
  ],
  era_narrow => [
    "p.n.e.",
    "n.e.",
  ],
  era_wide => [
    "prije nove ere",
    "nove ere",
  ],
  first_day_of_week => 1,
  glibc_date_1_format => "%a %b %e %H:%M:%S %Z %Y",
  glibc_date_format => "%m/%d/%y",
  glibc_datetime_format => "%a %b %e %H:%M:%S %Y",
  glibc_time_12_format => "%I:%M:%S %p",
  glibc_time_format => "%H:%M:%S",
  language => "Serbian",
  month_format_abbreviated => [
    "jan.",
    "feb.",
    "mart",
    "apr.",
    "maj",
    "jun",
    "jul",
    "avg.",
    "sept.",
    "okt.",
    "nov.",
    "dec.",
  ],
  month_format_narrow => [
    "j",
    "f",
    "m",
    "a",
    "m",
    "j",
    "j",
    "a",
    "s",
    "o",
    "n",
    "d",
  ],
  month_format_wide => [
    "januar",
    "februar",
    "mart",
    "april",
    "maj",
    "jun",
    "jul",
    "avgust",
    "septembar",
    "oktobar",
    "novembar",
    "decembar",
  ],
  month_stand_alone_abbreviated => [
    "jan.",
    "feb.",
    "mart",
    "apr.",
    "maj",
    "jun",
    "jul",
    "avg.",
    "sept.",
    "okt.",
    "nov.",
    "dec.",
  ],
  month_stand_alone_narrow => [
    "j",
    "f",
    "m",
    "a",
    "m",
    "j",
    "j",
    "a",
    "s",
    "o",
    "n",
    "d",
  ],
  month_stand_alone_wide => [
    "januar",
    "februar",
    "mart",
    "april",
    "maj",
    "jun",
    "jul",
    "avgust",
    "septembar",
    "oktobar",
    "novembar",
    "decembar",
  ],
  name => "Serbian Montenegro Latin",
  native_language => "srpski",
  native_name => "srpski Crna Gora latinica",
  native_script => "latinica",
  native_territory => "Crna Gora",
  native_variant => undef,
  quarter_format_abbreviated => [
    "K1",
    "K2",
    "K3",
    "K4",
  ],
  quarter_format_narrow => [
    "1.",
    "2.",
    "3.",
    "4.",
  ],
  quarter_format_wide => [
    "prvi kvartal",
    "drugi kvartal",
    "tre\N{U+0107}i kvartal",
    "\N{U+010d}etvrti kvartal",
  ],
  quarter_stand_alone_abbreviated => [
    "K1",
    "K2",
    "K3",
    "K4",
  ],
  quarter_stand_alone_narrow => [
    "1.",
    "2.",
    "3.",
    "4.",
  ],
  quarter_stand_alone_wide => [
    "prvi kvartal",
    "drugi kvartal",
    "tre\N{U+0107}i kvartal",
    "\N{U+010d}etvrti kvartal",
  ],
  script => "Latin",
  territory => "Montenegro",
  time_format_full => "HH:mm:ss zzzz",
  time_format_long => "HH:mm:ss z",
  time_format_medium => "HH:mm:ss",
  time_format_short => "HH:mm",
  variant => undef,
  version => 33,
}
